SystemOrganization addCategory: #'People-spfa-files'!
SystemOrganization addCategory: #'People-spfa-files-Tests'!


TestCase subclass: #FileReferenceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'People-spfa-files-Tests'!

FileList2 subclass: #FileBrowser
	instanceVariableNames: 'suffixes'
	classVariableNames: 'LastDirectories RecentDirectories'
	poolDictionaries: ''
	category: 'People-spfa-files'!

FileBrowser class
	instanceVariableNames: ''!

FileReferenceTest class
	instanceVariableNames: ''!

Object subclass: #FileReference
	instanceVariableNames: 'string lastAccessTime contents'
	classVariableNames: 'Files TmpFileSemaphore'
	poolDictionaries: ''
	category: 'People-spfa-files'!

FileReference class
	instanceVariableNames: ''!


!FileBrowser commentStamp: 'spfa 5/16/2003 15:45' prior: 0!
Improved FileList2 : this one remembers the directories from where files were picked. those directories are stored in different lists, according to the files extensions

	"SPFAFileList modalFileSelectorForSuffixes: #('mid')"!
]style[(224)!

!FileReference commentStamp: 'spfa 11/13/2003 15:19' prior: 0!
Interface for referencing a file.

The idea is to work with files (considered mainly as data external to the image and as external programs binary locations) in a simple way.

some properties:

	- FileReferences are unique for any file name: they can not be copied. 

	- a FileReference store a lastAccessTime inst var allowing to detect if the file was modified on the disk by an external process.

	- a FileReference can also record the file content, enabling an image or project to swallow all externally referenced data so that it can be ported to another system.

	- there is support for temporary files (automatic naming and removal, periodic cleaning of garbage on the disk)
	
!
]style[(684)!

!FileReferenceTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class FileReference. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category
!
]style[(362)!

!FileBrowser methodsFor: 'services' stamp: 'spfa 11/18/2009 18:09'!
addPath: aString
	"Add the given string to the list of recently visited directories."

	aString ifNil: [^self].

	"If a sub-directory is in the list, do nothing."
	(RecentDirectories detect: [ :aDir | ((aString, '*') match: aDir)] ifNone: [nil])
		ifNotNil: [^self].

	[RecentDirectories size >= 20]
		whileTrue: [RecentDirectories removeFirst].
	RecentDirectories addLast: aString! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
alterEgo

	^ FileReference allInstances 
		detect: [:f | [f entry = self entry] on: Error do: [^ self]]
		ifNone: [^ self]
! !

!FileReference methodsFor: 'converting' stamp: 'spfa 3/11/2010 11:45'!
asByteArray

	^ self fileStream binary contents
! !

!FileReference methodsFor: 'converting' stamp: 'spfa 5/2/2012 11:22'!
asFileNameWithDulySeparator

	^ self asString asFileNameWithDulySeparator
! !

!String methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:16'!
asFileNameWithDulySeparator

	| fname |

	fname := ((self copyReplaceAll: '/' with: FileDirectory slash)
		copyReplaceAll: '\' with: FileDirectory slash).
		"copyReplaceAll: ':' with: FileDirectory slash" 

"the last line was supposed to handle mac separators, but it clashes with Windows c:\ like path names"

	(FileDirectory slash == $:) ifTrue: "mac OS"
	[
		fname first == $: ifTrue: [fname := fname allButFirst].
		self first isAlphaNumeric ifTrue: [fname := ':', fname].
	].

	^ fname
! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
asFileReference

	^ self! !

!String methodsFor: '*People-spfa-files' stamp: 'spfa 4/29/2012 12:06'!
asFileReference

	^ FileReference for: self asFileNameWithDulySeparator! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
asString

	^ self fileName! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
asUrl

	^ FileUrl pathParts: (self directory pathParts copyWith: self localName)! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
baseName
		
	^ FileDirectory baseNameFor: self fileName! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
binaryOverwriteWith: aString

	(CrLfFileStream fileNamed: self asString) binary nextPutAll: aString; close.

! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
cleanUpFiles

	Preferences alwaysRemoveTemporaryFiles ifTrue:
		[self files keysAndValuesDo: [:k :v | 
			(v isNil and: [self isTmp: k])
				ifTrue: [
					MuOLog log: ('Deleting temporary file %Q' sformat: k).
					(FileDirectory forFileName: k) deleteFileNamed: k
				]]].

	self files keysAndValuesRemove: [:k :v | v isNil]! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
clone
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 11/18/2009 18:09'!
contentsOf: aFile

	^ aFile asFileReference fileContents! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
copy
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
copyAs: aFileReference

	| file1 file2 |
	file1 := (self directory readOnlyFileNamed: self asString) binary.
	file2 := (self directory fileNamed: aFileReference asString) binary.
	self directory copyFile: file1 toFile: file2.
	file1 close.
	file2 close.

	^ aFileReference! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
create

	self exists ifFalse:
		[(CrLfFileStream fileNamed: self asString) close].
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
createFileRootname: aString ext: extension inDirectory: dir

	self tmpSema critical:
	[
		| n file files | 

		files := dir fileNames.

		n := 100000000000 atRandom.
		[
			file := aString, n asString, (FileDirectory dot), extension.
			files includes: file
		]
		whileTrue: [n := 100000000000 atRandom].
					
		^ (FileReference for: dir pathName, (FileDirectory slash), file) create. 
		"otherwise that name may be attributed again !!"
	]


! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
csdFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
csdString

	^ self fileContents! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
deepCopy
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 11/18/2009 18:09'!
deleteFile: aFile

	aFile asFileReference kill! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
dirPath
		
	^ FileDirectory dirPathFor: self asString! !

!FileReference methodsFor: 'about' stamp: 'spfa 9/6/2013 09:09'!
dirPathTokens
		
	^ self dirPath findTokens: FileDirectory pathNameDelimiter! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
directory
		
	^ FileDirectory forFileName: self asString! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
displayContents

	self exists ifTrue:
		[self fileContents displayWithLabel: self asString]
! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
edit

	self exists ifFalse: [^ self].

	(Preferences editFilesWithEmacs
		and: [EmacsInteraction isAvailable]
		and: [EmacsInteraction wantsToEditFile: self])
	ifTrue:
		[^ EmacsInteraction editFile: self].

	FileList openEditorOn: self fileStream editString: nil! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 11/18/2009 18:09'!
editFile: aFile

	aFile asFileReference edit! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFileExists: aFileName

	FileReference ensureFileExists: aFileName 
		  error: (aFileName asString, ' does not exist')! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFileExists: aFileName else: aBlock

	(self fileExists: aFileName)
		ifFalse: [^ aBlock value].
! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFileExists: aFileName elseWrite: aString

	FileReference ensureFileExists: aFileName 
		  else: [self writeFile: aFileName withContents: aString]! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFileExists: aFileName error: aString

	(self fileExists: aFileName)
		ifFalse: [self error: aString].
! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFileExists: aFileName error: aString thenWait: millisecs

	FileReference ensureFileExists: aFileName error: aString.

	(Delay forMilliseconds: millisecs) wait.! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/18/2009 18:09'!
ensureFilesExist: aSetOfFileName

	aSetOfFileName do: [:ea | self ensureFileExists: ea].
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
entry 

	^ FileDirectory directoryEntryFor: self asString! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 11/18/2009 18:09'!
executableExtensions

	"we should be able to extend this list easily..."

	^ #('exe' 'bat' 'com')
		

! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
exists

	^ self class fileExists: self fileName! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
existsOrFails

	self class ensureFileExists: self fileName.! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
existsOrFails: aString

	self class ensureFileExists: self fileName error: aString.! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
extension
		
	^ FileDirectory extensionFor: self fileName! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 9/13/2011 22:31'!
fileContents

	self isInlined ifTrue: [^ contents copy].

	^ (CrLfFileStream readOnlyFileNamedSansConcreteStream: self asString)
		contentsOfEntireFile.! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 4/29/2012 12:07'!
fileExists: aFileName 

	^ (self fileNamed: aFileName asFileNameWithDulySeparator) notNil
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
fileName

	"always return a full name !!"

	^ FileDirectory default fullNameFor: 
		(string ifNil: [^ '']) asFileNameWithDulySeparator! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/18/2009 18:09'!
fileName: aString

	string := aString.
	lastAccessTime := nil.
	self forgetContents! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 4/29/2012 12:07'!
fileNamed: aFileName 

	^ (FileDirectory forFileName: aFileName asString) 
		entryAt: (FileDirectory localNameFor: aFileName asString)
		ifAbsent: [nil]
	
! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 4/20/2010 17:08'!
fileSize: aFileName 

	^ (self fileNamed: aFileName) fileSize
	
! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 3/11/2010 11:12'!
fileStream

	self exists ifFalse: [self error: 'this file does not exists'].
	^ FileDirectory default fileNamed: self asString

	! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
files

	^ Files ifNil: [Files := WeakValueDictionary new]! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 2/7/2014 16:36'!
for: aFileName

	| actualName |
	
	actualName := aFileName withoutQuoting withoutTrailingBlanks.
"
	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: [actualName := actualName asLowercase].
"
	((self files includesKey: actualName)
		and: [(self files at: actualName) notNil])
		ifTrue: [^ self files at: actualName]
		ifFalse: [ | fref | 
					fref := self new fileName: actualName.
					self files add: actualName -> fref.
					^ fref]
! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 11/18/2009 18:09'!
forgetContents

	contents := nil
	

	! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
fullExecutableName

	self possibleExecutableNames
		do: [:n | n asFileReference exists ifTrue: [^ n asFileReference fullName]].
		
	(self dirPath = '')
	ifTrue: 
		[^ (FileReference path collect: [:p |
			(self fullNameIn: (FileDirectory on: p)) asFileReference fullExecutableName])
			detect: [:n | n notNil] ifNone: [nil]].

	^ nil! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
fullName

	^ self fullNameIn: FileDirectory default
! !

!FileReference methodsFor: 'about' stamp: 'spfa 2/5/2012 17:51'!
fullNameFor: aString

	^ (FileDirectory forFileName: self asString, FileDirectory slash) fullNameFor: aString
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
fullNameIn: aFileDirectory

	^ aFileDirectory fullNameFor: self asString
! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 11/18/2009 18:09'!
getEnv: aString

	| var |

	OSProcess accessor canAccessSystem
		ifFalse: [self error: 'OSProcess unavailable'].

	var := aString asUppercase.

	OSProcess thisOSProcess environment associationsDo:
		[:a | a key asUppercase = var
			ifTrue: [^ a value veryDeepCopy]].

	^ nil

! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
initialize

	"SPFAFileList initialize"

	super initialize.

	LastDirectories := Dictionary new.
	RecentDirectories := OrderedCollection new.
	! !

!FileReference class methodsFor: 'initialization' stamp: 'spfa 11/18/2009 18:09'!
initialize

	self registerStringFormats
	! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isExecutable

	"note: this is different from OSProcess accessor isExecutable: aString"
	
	"this is quite slow. better avoid using this method at all..."

	(self dirPath = '')
	ifFalse:
	[
		self possibleExecutableNames
			do: [:n | ([FileDirectory default fileExists: n] on: Error do: [false]) 
						ifTrue: [^ true]].		
	]
	ifTrue: 
		[FileReference path do: 
			[:p |
				 (self fullNameIn: (FileDirectory on: p)) asFileReference isExecutable
			 	 ifTrue: [^ true]]].

	^ false
! !

!FileReference class methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isExecutable: aString

	"note: this is different from OSProcess accessor isExecutable: aString"

	aString ifNil: [^ false].

	{ aString . aString, '.bat' . aString, '.exe' . aString, '.com'}
	do: [:n | (self fileExists: n) ifTrue: [^ true]].
		
	(FileDirectory dirPathFor: aString) = ''
	ifTrue: 
		[self path do: 
			[:p |
				 (self isExecutable: ((FileDirectory on: p) fullNameFor: aString))
			 	 ifTrue: [^ true]]].

	^ false
! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isFile

	^ true! !

!Object methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:15'!
isFile

	^ false! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isFileReference

	^ true! !

!Object methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:15'!
isFileReference

	^ false! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isInlined

	^ contents notNil! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
isReferencedElsewhere

	"comment implmenter a au plus juste ??"
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
isTmp: aFile

	| tmpRoot |

	tmpRoot := self tmpFileNameRoot.
	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: [tmpRoot := tmpRoot asLowercase].
	^ (FileDirectory localNameFor: aFile asString) beginsWith: tmpRoot! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
kill

	self exists
	ifTrue:
		[self directory deleteFileNamed: self asString].
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
lastAccessTime

	^ 
lastAccessTime ! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
lastDirectories

	^ LastDirectories 
	! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
localName
		
	^ FileDirectory localNameFor: self fileName! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
localNameSansExtension
		
	^ FileDirectory baseNameFor: (FileDirectory localNameFor: self fileName)! !

!FileReference methodsFor: 'about' stamp: 'spfa 1/26/2010 23:46'!
localNameSansVersionNumber
		
	| n i |

	n := self localNameSansExtension.
	i := n lastIndexOf: $. ifAbsent: [nil].
	(i notNil and: [(n copyFrom: i + 1 to: n size) isAllDigits])
		ifTrue:	[n := n copyFrom: 1 to: i - 1].

	^ n! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
midiFile

	^ self ! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
modalFileSelectorForSuffixes: aList inDir: aDirectory

	| window aFileList |

	window := self morphicViewFileSelectorForSuffixes: aList inDir: aDirectory.
	aFileList := window valueOfProperty: #fileListModel.
	window openCenteredInWorld.
	[window world notNil] whileTrue: [
		window outermostWorldMorph doOneCycleNow.
	].
	^aFileList getSelectedFile! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
modificationTime 

	^ self entry modificationTime! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
morphicViewFileSelectorForSuffixes: aList

	^ self morphicViewFileSelectorForSuffixes: aList inDir: nil! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 4/21/2011 23:29'!
morphicViewFileSelectorForSuffixes: aList inDir: aDirectory
	"Answer a morphic file-selector tool for the given suffix list"

	| dir aFileList window fixedSize midLine gap |

	aDirectory ifNotNil: [dir := aDirectory]
	ifNil:
	[

	aList 
	ifNil: [dir := FileDirectory default]
	ifNotNil: 
		[dir := LastDirectories at: aList 
									  ifAbsent: [FileDirectory default]].
	].
	aFileList := self new directory: dir.
	aFileList suffixes: aList.

	aFileList optionalButtonSpecs: aFileList okayAndCancelServices.
	aList ifNotNil:
		[aFileList fileSelectionBlock: [:entry :myPattern |
			entry isDirectory
				ifTrue:
					[false]
				ifFalse:
					[aList includes: (FileDirectory extensionFor: entry name asLowercase)]] fixTemps].
	window := BorderedMorph new
		layoutPolicy: ProportionalLayout new;
		color: Color tan muchLighter;
		borderColor: Color gray;
		borderWidth: 2;
		layoutInset: 4;
		extent: 600@400;
		useRoundedCorners.
	window setProperty: #fileListModel toValue: aFileList.
	aFileList modalView: window.
	midLine := 0.4.
	fixedSize := 25.
	gap := 1.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a file'. 0 @ 0 corner: 1 @ 0. 0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 
					gap @(fixedSize * 2) corner: gap negated@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 
					gap@(fixedSize * 2) corner: gap negated@0}.
	}.


	aFileList postOpen.
	aFileList setSelectedDirectoryTo: dir.
	aFileList resort: #name.

	^ window ! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 4/3/2011 15:18'!
named: aString inSubDirectory: anArray

	| dir |

	dir := FileDirectory default.
	anArray do: [:s | dir := dir directoryNamed: s].

	^ FileReference for: (FileReference for: (dir fullNameFor: aString)) relativeName! !

!FileReference methodsFor: 'about' stamp: 'spfa 1/26/2010 22:46'!
nextFile

	^ (self directory fullNameFor: self nextLocalName) asFileReference! !

!FileReference methodsFor: 'about' stamp: 'spfa 1/26/2010 23:46'!
nextLocalName

	^ self directory nextNameFor: self localNameSansVersionNumber
		 extension: self extension! !

!FileReference methodsFor: 'about' stamp: 'spfa 9/14/2011 11:09'!
nextName

	^ self directory plainNextNameFor: self baseName extension: self extension! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
noteAccessTime

	
lastAccessTime := Date today asSeconds + Time now asSeconds! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/18/2009 18:09'!
objectForDataStream: refStrm
    "Return an object to store on an external data stream."

	"  revoir compltement"
	
	^ self 
"
    self isInlined ifTrue: [^ self]
		ifFalse: [(self confirm: 'do we inline ', self fullName, ' ?')
				 ifTrue: [^ self swallowContents]
				 ifFalse: [^ self]]
"! !

!FileBrowser methodsFor: 'hits' stamp: 'spfa 11/18/2009 18:09'!
okHit
	
	self class lastDirectories add: (suffixes -> self directory).
	self addPath: directory pathName.
	super okHit! !

!FileBrowser methodsFor: 'services' stamp: 'spfa 11/18/2009 18:09'!
okayAndCancelServices
	"Answer ok and cancel services"

	^ {self serviceOkay. self serviceRecentDirs . self serviceCancel}! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
orchestraFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
orchestraString

	^ self fileContents! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/18/2009 18:09'!
overwriteWith: aString

	(CrLfFileStream fileNamed: self asString) nextPutAll: aString; close.

! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 11/18/2009 18:09'!
path

	| path |

	path := self getEnv: 'PATH'.
	path ifNil: [self error: 'no PATH detected !!'].

	^ ((path replaceAll: $ with: $&) replaceAll: $; with: $ ) substrings
		collect: [:d | d replaceAll: $& with: $ ]

		

! !

!FileDirectory class methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:09'!
pathParts: anOrderedCollection

	^ self on: (anOrderedCollection inject: '' into: [:path :part  | 
						path, part , self pathNameDelimiter asString])! !

!FileDirectory methodsFor: '*People-spfa-files' stamp: 'spfa 9/14/2011 11:08'!
plainNextNameFor: baseFileName extension: extension
	"Assumes a file name includes a version number encoded as '.' followed by digits 
	preceding the file extension.  Increment the version number and answer the new file name.
	If a version number is not found, set the version to 1 and answer a new file name"

	| files splits version |

	files := self fileNamesMatching: (baseFileName,'*', self class dot, extension).
	splits := files 
			collect: [:file | self splitNameVersionExtensionFor: file]
			thenSelect: [:split | (split at: 1) = baseFileName].
	splits := splits asSortedCollection: [:a :b | (a at: 2) < (b at: 2)].
	splits isEmpty 
			ifTrue: [version := 1]
			ifFalse: [version := (splits last at: 2) + 1].
	^ (baseFileName, '.', version asString, self class dot, extension)! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/18/2009 18:09'!
possibleExecutableNames

	^ {self fileName}, (self class executableExtensions collect: [:x |
		FileDirectory fileName: self fileName extension: x])! !

!FileReference methodsFor: 'printing' stamp: 'spfa 11/18/2009 18:09'!
printOn: aStream

	string 
		ifNil: [aStream nextPutAll: '[-void-]']
		ifNotNil: [aStream nextPut: $[; nextPutAll: (string ifNil: ['']) asFileNameWithDulySeparator; nextPut: $]] 
! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
query: aString

	"FileReference query: 'file name ?'"

	^ self query: aString directory: FileDirectory default
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
query: aString directory: aDirectory

	^ self query: aString directory: aDirectory default: ''
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
query: aString directory: aDirectory default: bString

	^ (((FileBrowser modalFolderSelector: aDirectory) ifNil:[^ nil]) pathName,
			FileDirectory slash,
	    		((FillInTheBlank request: aString initialAnswer: bString) 
				whenEmpty: [^ nil]))
		asFileReference
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 11/18/2009 18:09'!
query: aString for: receiver getter: gSelector setter: sSelector

	| ddir dname got |

	got := receiver perform: gSelector.
	ddir := got ifNil: [FileDirectory default] ifNotNil: [got asFileReference directory].
	dname := got ifNil: [''] ifNotNil: [got asFileReference localName].

	receiver perform: sSelector withArguments: 
		{(self query: aString directory: ddir default: dname) ifNil: [^ self]}
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 1/26/2010 22:25'!
query: aString inDirectory: aDirectory default: bString

	^ ((aDirectory ifNil:[^ nil]) pathName,
			FileDirectory slash,
	    		((FillInTheBlank request: aString initialAnswer: bString) 
				whenEmpty: [^ nil]))
		asFileReference
								! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/18/2009 18:09'!
readDataFrom: aDataStream size: varsOnDisk
   
	super readDataFrom: aDataStream size: varsOnDisk.

	self isInlined ifTrue:
	[
		(self confirm: 'do we create ', self fullName, ' ?')
				 ifTrue: [self restoreContents]
	].

	self forgetContents.

	^ self alterEgo
! !

!CrLfFileStream class methodsFor: '*people-spfa-files' stamp: 'spfa 9/13/2011 22:31'!
readOnlyFileNamedSansConcreteStream: fileName 
	"Open an existing file with the given name for reading."

	| fullName f |
	fullName := self fullName: fileName.
	f := self basicNew "skip what happens in #new" open: fullName forWrite: false.
	^ f isNil
		ifFalse: [f]
		ifTrue: ["File does not exist..."
			((FileDoesNotExistException fileName: fullName) readOnly: true) signal].
! !

!FileBrowser methodsFor: 'hits' stamp: 'spfa 11/18/2009 18:09'!
recentDirsHit
	"Put up a menu and let the user select from the list of recently visited directories."

	| dirName |
	RecentDirectories isEmpty ifTrue: [^self].
	dirName := (SelectionMenu selections: RecentDirectories) startUp.
	dirName == nil ifTrue: [^self].
	self setSelectedDirectoryTo: (FileDirectory on: dirName).

! !

!FileReference class methodsFor: 'initialization' stamp: 'spfa 11/18/2009 18:09'!
registerStringFormats

	Smalltalk at: #StringFormat ifAbsent: [^ self].
	StringFormat replacementsDictionary  
		add: $F -> [:o :n | FileReference wrapFileName: o asString];
		add: $E -> [:o :n | FileReference wrapFileNameForEmacs: o asString].
	StringFormat helpDictionary  
		add: '%F' -> 'file reference';
		add: '%E' -> 'file reference for Emacs'.! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
relativeName

	"truly relative name, do not confuse with FileDirectory>>relativeNameFor: which does the opposite as what its name says"

	^ self relativeNameFor: FileDirectory default
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/18/2009 18:09'!
relativeNameFor: aDirectory

	"truly relative name, do not confuse with FileDirectory>>relativeNameFor: which does the opposite as what its name says"

	| pp dpp delim |

	pp := self directory pathParts.
	dpp := aDirectory pathParts.
	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: 
		[
			pp := pp collect: [:p | p  asLowercase]. 	
			dpp := dpp collect: [:p | p  asLowercase]. 	
		].
	delim := FileDirectory default pathNameDelimiter asString.

	(pp beginsWith: dpp)
		ifTrue: [^ ((pp allButFirst: dpp size) inject: '' into: [:name :p |
							name, p, delim]),
					self localName].

	^ '..', delim, (self relativeNameFor: aDirectory upperDirectory)
! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 2/5/2012 21:08'!
renameFile: aFile

	| filename |

	filename := UIManager default
		request: 'New File Name?' 
		initialAnswer: aFile fileName.

	filename ifEmpty: [^ aFile].

	^ aFile renamedAs: filename asFileReference
! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
renamedAs: aFile

	aFile exists ifTrue: [aFile kill].
	Smalltalk garbageCollect.
	self directory rename: self asString toBe: aFile asString.

	^aFile! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 3/11/2010 11:08'!
restoreContents

	self isInlined ifFalse: [self error: 'no inlined contents'].
	
	((FileReference fileExists: self fullName) 
		and: [(self confirm: self fullName, ' already exists. Overwrite it ?') not])
	ifTrue: [^ self].

	(FileDirectory default directoryExists: self dirPath)
		ifFalse:
			[ "the path does not exists; change to FileDirectory default"
				string := self localName.
			].

	self binaryOverwriteWith: contents.
	self forgetContents


	! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
scoreFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
scoreString

	^ self fileContents! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
seemsToBeATmpFile

	^ self class isTmp: self! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 11/18/2009 18:09'!
selectDirectory

	^ FileBrowser modalFolderSelector
! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 11/18/2009 18:09'!
selectFile

	| file |

	(file := FileBrowser modalFileSelector) ifNil: [^nil].
	file close.

	^ file name asFileReference.
! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 11/18/2009 18:09'!
selectFile: aList

	^ self selectFile: aList inDir: nil	! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 11/18/2009 18:09'!
selectFile: aList inDir: aDirectory

	| file |

	(file := FileBrowser modalFileSelectorForSuffixes: aList inDir: aDirectory) ifNil: [^nil].
	file close.

	^ file name asFileReference.
! !

!FileBrowser methodsFor: 'services' stamp: 'spfa 11/18/2009 18:09'!
serviceRecentDirs 

	"Answer a service for quick selection of recently used directories"

	^ (SimpleServiceEntry new
		provider: self label: 'recentDirs' selector: #recentDirsHit 
		description: 'hit here to pop up the Recent Directories menu')
		extraSelector: nil;
		buttonLabel: 'recent dirs'! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
size

	^ self class fileSize: self fileName! !

!FileBrowser methodsFor: 'suffixes' stamp: 'spfa 11/18/2009 18:09'!
suffixes: aList

	suffixes := aList! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 3/11/2010 11:59'!
swallowContents

	self noteAccessTime.
	contents := self asByteArray
	
	

	! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
testFileExists

	self assert: (FileReference fileExists: SmalltalkImage current imageName).
	self deny: (FileReference fileExists: 'some dummy name.yop').
				  ! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
testStringFormat

	self assert: ('%s -W -r %s -o %F' sformat: {'srconv' . 44100 . 'hi joe.wav'}) ='srconv -W -r 44100 -o "hi joe.wav"' 
				 ! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:09'!
testTmpFile

	| tmpFile |

	self assert: ((FileReference withTmpFileDo: [:f | 
					tmpFile := f.
					f overwriteWith: 'yo'.
					f fileContents]) = 'yo'
				 and: [tmpFile exists not] )
				 ! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 11/18/2009 18:09'!
tmpDirectory

	| dir |

	dir := ('tmp', FileDirectory slash) asFileReference directory. 
	dir exists ifFalse: [FileDirectory default createDirectory: 'tmp'].

	^ dir! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpFileExt: extension

	^ self tmpFileExt: extension inDirectory: self tmpDirectory.! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpFileExt: extension inDirectory: dir

	^ self createFileRootname: self tmpFileNameRoot ext: extension inDirectory: dir
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpFileExt: extension inSameDirectoryAs: aFileName

	^ self tmpFileExt: extension inDirectory: (FileDirectory forFileName: aFileName asString)

! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpFileNameExtension

	^ 'TMP'
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpFileNameRoot

	^ '_TMP_'! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
tmpSema

	^ TmpFileSemaphore ifNil: 
		[TmpFileSemaphore := Semaphore forMutualExclusion]! !

!FileDirectory methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:09'!
upperDirectory

	^ self class pathParts: self pathParts allButLast! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/18/2009 18:09'!
veryDeepCopyWith: deepCopier
	"Return self.  I can't be copied."! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/18/2009 18:09'!
wasModified

	^ (lastAccessTime isNil
			or: [self entry isNil]
			or: [self entry modificationTime > lastAccessTime])! !

!FileReference methodsFor: 'converting' stamp: 'spfa 1/26/2010 23:02'!
withExtension: aString

	^ (self baseName, FileDirectory dot, aString) asFileReference! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
withTmpFileDo: aBlock

	^ self withTmpFileExt: self tmpFileNameExtension do: aBlock.
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
withTmpFileExt: anExtension do: aBlock

	^ self withTmpFileExt: anExtension inDirectory: self tmpDirectory do: aBlock.
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
withTmpFileExt: anExtension inDirectory: dir do: aBlock

	| tmpFileRef returnValue |

	tmpFileRef := FileReference tmpFileExt: anExtension inDirectory: dir.
	[returnValue := aBlock value: tmpFileRef]

		"DEBUG:
					 tmpFileRef displayContents 
		"

		ensure: [tmpFileRef kill].

	^ returnValue! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/18/2009 18:09'!
withTmpFileExt: anExtension inSameDirectoryAs: aFileReference do: aBlock

	^ self withTmpFileExt: anExtension inDirectory: aFileReference directory do: aBlock.

! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/18/2009 18:09'!
wrapFileName

	^ FileReference wrapFileName: self fileName! !

!FileReference class methodsFor: 'utils' stamp: 'spfa 11/18/2009 18:09'!
wrapFileName: aFileName

	((aFileName includes: $ )
		and: [(aFileName includes: $") not])
	ifTrue: [^ '"', aFileName, '"'].

	^ aFileName.! !

!FileReference class methodsFor: 'utils' stamp: 'spfa 11/18/2009 18:09'!
wrapFileNameForEmacs: aFileName

	^ '"', (aFileName copyReplaceAll: '\' with: '/'), '"'! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 11/18/2009 18:09'!
writeFile: aFile withContents: aString 

	^ aFile asFileReference overwriteWith: aString

	! !

!String methodsFor: '*People-spfa-files' stamp: 'spfa 11/18/2009 18:16'!
writeToFile: aFileReference

	aFileReference overwriteWith: self! !
FileBrowser initialize!
FileReference initialize!
