


!SimpleMIDIPort methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/21/2013 11:25'!
bufferTimeStampFrom: aByteArray
	"Return the timestamp from the given MIDI input buffer. Assume the given buffer is at least 4 bytes long."


	^ ((aByteArray at: 4) bitShift: 24) +
	  ((aByteArray at: 3) bitShift: 16) +
	  ((aByteArray at: 2) bitShift: 8) +
	   (aByteArray at: 1)
! !

!TestBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 10/13/2011 11:19'!
classList

	"hack for compatibility after Browser refactoring in Squeak 4.3"
	(self class canUnderstand: #selectedSystemCategory)
		ifTrue: [^ super classList].

	^systemCategoryListIndex = 0
		ifTrue: [Array new]
		ifFalse: [self classListAtCategoryNamed: (self systemCategoryList at: systemCategoryListIndex)]! !

!Color class methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 10/3/2011 21:59'!
colorFromPixelValue: p depth: d
    "Convert a pixel value for the given display depth into a color."
    "Details: For depths of 8 or less, the pixel value is simply looked 
up in a table. For greater depths, the color components are extracted 
and converted into a color."
    "Warning: In BitBlt, a pixel with pixelValue = 0 is transparent.
    Squeak usually assumes that r=g=b=0 => transparent. But this is 
false if we have alpha (opacity).
    A color with r=g=b=0 and opacity = 255 is BLACK, not TRANSPARENT.
    Squeak also answers darkest possible blue when asked for black. 
Again, this is not needed in 32 bits (with alpha).
    The real rule is that pixelValue=0 means transparent.
    And that darkest blue must be used instead of black, but only for 
depths >8 and < 32 (no indexed colors, no alpha)
    This method is updated to reflect that."

    | r g b alpha |
    d = 8 ifTrue: [^ IndexedColors at: (p bitAnd: 16rFF) + 1].
    d = 4 ifTrue: [^ IndexedColors at: (p bitAnd: 16r0F) + 1].
    d = 2 ifTrue: [^ IndexedColors at: (p bitAnd: 16r03) + 1].
    d = 1 ifTrue: [^ IndexedColors at: (p bitAnd: 16r01) + 1].

    d = 32 ifTrue: [
        "eight bits per component; 8 bits of alpha"
        alpha := p bitShift: -24.
        alpha = 0 ifTrue: [ ^Color transparent ].
        r := (p bitShift: -16) bitAnd: 16rFF.
        g := (p bitShift: -8) bitAnd: 16rFF.
        b := p bitAnd: 16rFF.
        ^alpha < 255
            ifTrue: [ (Color r: r g: g b: b range: 255) alpha: alpha 
asFloat / 255.0 ]
            ifFalse: [ Color r: r g: g b: b range: 255 ]].

    "For the rest of the depths, pixelValue = 0 means transparent, and 
darkest blue is considered to be black."
    p = 0 ifTrue: [ ^Color transparent ].

    (d = 16) | (d = 15) ifTrue: [
        "five bits per component"
        r := (p bitShift: -10) bitAnd: 16r1F.
        g := (p bitShift: -5) bitAnd: 16r1F.
        b := p bitAnd: 16r1F.
        (r = 0 and: [ g = 0 and: [ b = 1]]) ifTrue: [
            ^Color black ].
        ^ Color r: r g: g b: b range: 31].

    d = 12 ifTrue: [
        "four bits per component"
        r := (p bitShift: -8) bitAnd: 16rF.
        g := (p bitShift: -4) bitAnd: 16rF.
        b := p bitAnd: 16rF.
        (r = 0 and: [ g = 0 and: [ b = 1]]) ifTrue: [
            ^Color black ].
        ^ Color r: r g: g b: b range: 15].

    d = 9 ifTrue: [
        "three bits per component"
        r := (p bitShift: -6) bitAnd: 16r7.
        g := (p bitShift: -3) bitAnd: 16r7.
        b := p bitAnd: 16r7.
        (r = 0 and: [ g = 0 and: [ b = 1]]) ifTrue: [
            ^Color black ].
        ^ Color r: r g: g b: b range: 7].

    self error: 'unknown pixel depth: ', d printString! !

!Morph methodsFor: '*People-spfa-fixes' stamp: 'spfa 8/11/2010 18:02'!
containingWindow
	"Answer a window or window-with-mvc that contains the receiver"

	| component |
	component := self.
	component model isNil ifTrue: [component := self firstOwnerSuchThat: [:m| m model notNil]].
	^ ((component isNil or: [component isWindowForModel: component model])
		ifTrue: [component]
		ifFalse: [component firstOwnerSuchThat:[:m| m isWindowForModel: component model]])
		ifNil: [self firstOwnerSuchThat: [:m| m isSystemWindow]].! !

!Duration class methodsFor: '*People-spfa-fixes-override' stamp: 'brp 1/7/2004 15:38'!
days: days hours: hours minutes: minutes seconds: seconds nanoSeconds: nanos

 	^ self nanoSeconds: 
			( ( (days * SecondsInDay) 
				+ (hours * SecondsInHour)
					+ (minutes * SecondsInMinute) 
						+ seconds ) * NanosInSecond )
							+ nanos.
! !

!MethodNode methodsFor: '*People-spfa-fixes' stamp: 'spfa 4/4/2012 11:29'!
decompileString 
	"Answer a string description of the parse tree whose root is the receiver."

	^self fullPrintString
! !

!TestBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:17'!
defaultBrowserTitle
	"Returns the receivers default browser title"
	
	^self asString! !

!NCAAConnectorMorph class methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:14'!
defaultFSM

	^ nil! !

!Point methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 11/18/2009 18:15'!
dist: aPoint 
	"Answer the distance between aPoint and the receiver."

	| dx dy |

	dx := aPoint x - x.
	dy := aPoint y - y.

	^ ((dx * dx) + (dy * dy)) sqrt! !

!MorphicUIManager methodsFor: '*People-spfa-fixes' stamp: 'spfa 2/2/2012 11:51'!
edit: aText label: labelString accept: anAction
	"Open an editor on the given string/text"
	| window holder text |
	holder := Workspace  new.
	holder contents: aText.
	text := PluggableTextMorphPlus 
		on: holder 
		text: #contents 
		accept: #acceptContents: 
		readSelection: nil 
		menu: nil.
	text acceptAction: anAction.
	window := SystemWindow new.
	labelString ifNotNil:[window setLabel: labelString].
	window model: holder .
	window addMorph: text frame: (0@0 extent: 1@1).
	window openInWorld.
	^window! !

!SystemProgressMorph methodsFor: '*People-spfa-fixes' stamp: 'spfa 7/9/2010 12:03'!
freeSlot: number
	number > 0 ifTrue: [
		lock critical: [
			(bars at: number) delete.
			(labels at: number) delete.
			activeSlots := activeSlots - 1.
			activeSlots <= 0
				ifTrue: [self delete]
				ifFalse: [self recenter]]]! !

!Duration methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 11/18/2009 18:09'!
hash 	

	^ seconds asInteger bitXor: nanos asInteger
! !

!FileList2 class methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:09'!
initialize
	Preferences
		addPreference: #useFileList2
		categories:  #(general)
		default: true
		balloonHelp: 'if true, then when you open a file list from the World menu, it''ll be an enhanced one'
		projectLocal:  false
		changeInformee:  self
		changeSelector: #useFileList2preferenceChanged
		viewRegistry: PreferenceViewRegistry ofBooleanPreferences ! !

!FileContentsBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:09'!
isTestCase: aPseudoClass

	^ false! !

!TestBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 4/19/2010 15:08'!
layoutWithTestPane

	topWindowMorph removePanes.

	topWindowMorph 
		addMorph: self buildSystemCategoryListMorph frame: (0.0 @ 0.0 extent: 0.25 @ 0.23);
		addMorph: self buildTemplateMorph frame: (0.0 @ 0.23 extent: 0.25 @ 0.07);
"		addMorph: self buildToggleFilteringButtonMorph frame: (0.0 @ 0.23 extent: 0.12 @ 0.07);
		addMorph: self buildToggleTestPaneButtonMorph frame: (0.12 @ 0.23 extent: 0.13 @ 0.07);"
		addMorph: self buildClassListMorph frame: (0.25 @ 0.0 extent: 0.25 @ 0.30);
		addMorph: self buildMessageCategoryListMorph frame: (0.5 @ 0.0 extent: 0.25 @ 0.30);
		addMorph: self buildMessageListMorph frame: (0.75 @ 0.0 extent: 0.25 @ 0.30).

	topWindowMorph 
		addMorph: self buildRefreshButtonMorph frame: (0.0 @ 0.30 extent: 0.17 @ 0.12 "0.08");
		addMorph: self buildRunButtonMorph frame: (0.0 @ 0.42 "0.33" extent: 0.17 @ 0.17 "0.08");
		"addMorph: self buildStopButtonMorph frame: (0.0 @ 0.41 extent: 0.17 @ 0.08);"
		addMorph: self buildFailureButtonMorph frame: (0.17 @ 0.30 extent: 0.33 @ 0.07);
		addMorph: self buildFailuresListMorph frame: (0.17 @ 0.37 extent: 0.33 @ 0.22);
		addMorph: self buildErrorButtonMorph frame: (0.50 @ 0.30 extent: 0.33 @ 0.07);
		addMorph: self buildErrorListMorph frame: (0.50 @ 0.37 extent: 0.33 @ 0.22);
		addMorph: self buildRunCategoryButtonMorph frame: (0.83 @ 0.30 extent: 0.17 @ 0.08);
		addMorph: self buildRunClassButtonMorph frame: (0.83 @ 0.38 extent: 0.17 @ 0.08);
		addMorph: self buildRunMethodButtonMorph frame: (0.83 @ 0.46 extent: 0.17 @ 0.13).

	topWindowMorph 
		addMorph: self buildPassFailTextMorph frame: (0.0 @ 0.59 extent: 0.30 @ 0.07);
		addMorph: self buildDetailsTextMorph frame: (0.30 @ 0.59 extent: 0.70 @ 0.07).
	
	topWindowMorph 
		addMorph: self buildBrowserCodeMorph frame: (0.0 @ 0.66 extent: 1 @ 0.34).

	topWindowMorph setUpdatablePanesFrom: #(systemCategoryList classList messageCategoryList messageList).
	self refreshWindow.

	self isEmptySuite ifFalse: [self updateWindow].

	^ topWindowMorph! !

!ZipArchiveMember methodsFor: '*People-spfa-fixes-common' stamp: 'spfa 11/18/2009 18:17'!
localFileName
	"Answer my fileName in terms of the local directory naming convention"
	| localName |
	localName :=  (fileName copyReplaceAll: '/' with: FileDirectory slash)
		copyReplaceAll: '\' with: FileDirectory slash.
	^(fileName first = $/)
		ifTrue: [ FileDirectory default class makeAbsolute: localName ]
		ifFalse: [ FileDirectory default class makeRelative: localName ]! !

!ExternalDropHandler methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 11/18/2009 18:09'!
matchesExtension: aExtension
	(self extension isNil or: [aExtension isNil])
		ifTrue: [^false].
	FileDirectory activeDirectoryClass isCaseSensitive
		ifTrue: [^extension = aExtension]
		ifFalse: [^extension sameAs: aExtension]! !

!PolygonMorph methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:15'!
maybeBecomeNCLineMorph

	"for conversion from a connector v1.9 to a connector 2.2"

	| lw col o |

	(owner isNil or: [owner isConnector not]) ifTrue: [^ self].

	(owner isMGraphConnector and:
		[owner isTimeStamp]) ifTrue: [ lw := 1] ifFalse: [lw := 2].

	col := borderColor.
	o := owner.
	self privateOwner: nil.
	self become: ((NCLineMorph vertices: vertices lineWidth: lw lineColor: col borderWidth: 0 borderColor: col) privateOwner: o) 
		makeForwardArrow; fillStyle: col.

	owner beOrthogonal: false! !

!Duration class methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 11/18/2009 18:09'!
milliSeconds: milliCount

	^ self nanoSeconds: milliCount * 1000000
! !

!PluggableButtonMorph methodsFor: '*People-spfa-fixes' stamp: 'spfa 6/18/2013 11:17'!
mouseEnterDragging: evt

	self mouseEnter: evt! !

!Duration class methodsFor: '*People-spfa-fixes-override' stamp: 'spfa 11/18/2009 18:09'!
nanoSeconds: nanos

	| n s |

	n := nanos asInteger.
	^ self new
		seconds: (s := n quo: NanosInSecond) 
		nanoSeconds: (n - (s * NanosInSecond));
		yourself.
! !

!Object methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:15'!
perform: selector orSendTo: otherTarget
	"If I wish to intercept and handle selector myself, do it; else send it to otherTarget"
	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!Color methodsFor: '*people-spfa-fixes-override' stamp: 'spfa 10/3/2011 21:59'!
pixelValueForDepth: d
    "Returns an integer representing the bits that appear in a single 
pixel of this color in a Form of the given depth. The depth must be one 
of 1, 2, 4, 8, 16, or 32. Contrast with pixelWordForDepth: and 
bitPatternForDepth:, which return either a 32-bit word packed with the 
given pixel value or a multiple-word Bitmap containing a pattern. The 
inverse is the class message colorFromPixelValue:depth:"
    "Details: For depths of 8 or less, the result is a colorMap index. 
For depths of 16 and 32, it is a direct color value with 5 or 8 bits per 
color component."
    "Transparency: The pixel value zero is reserved for transparent. For 
depths greater than 8 and less than 32 (no Indexed colors, no real 
alpha), black maps to the darkest possible blue.
    Note that
        Color transparent class = TranslucentColor
    this special case is handled in TranslucentColor >> #pixelValueForDepth:
    "

    | bitBltFakeBlack val |
    d = 8 ifTrue: [^ self closestPixelValue8].  "common case"
    d < 8 ifTrue: [
        d = 4 ifTrue: [^ self closestPixelValue4].
        d = 2 ifTrue: [^ self closestPixelValue2].
        d = 1 ifTrue: [^ self closestPixelValue1]].

    d = 32 ifTrue: [
        "eight bits per component; top 8 bits set to all ones (opaque 
alpha)"
        val := LargePositiveInteger new: 4.
        val at: 3 put: ((rgb bitShift: -22) bitAnd: 16rFF).
        val at: 2 put: ((rgb bitShift: -12) bitAnd: 16rFF).
        val at: 1 put: ((rgb bitShift: -2) bitAnd: 16rFF).
        val at: 4 put: 16rFF.  "opaque alpha"
        ^ val normalize].

    "For the rest of the depths, pixelValue = 0 means transparent, and 
darkest blue is considered to be black."
    bitBltFakeBlack := 1.  "closest black that is not transparent in RGB 
- Not for depths <=8 (Indexed) or = 32 (RGBA)"

    d = 16 ifTrue: [
        "five bits per component; top bits ignored"
        val := (((rgb bitShift: -15) bitAnd: 16r7C00) bitOr:
             ((rgb bitShift: -10) bitAnd: 16r03E0)) bitOr:
             ((rgb bitShift: -5) bitAnd: 16r001F).
        ^ val = 0 ifTrue: [bitBltFakeBlack] ifFalse: [val]].

    d = 12 ifTrue: [  "for indexing a color map with 4 bits per color 
component"
        val := (((rgb bitShift: -18) bitAnd: 16r0F00) bitOr:
             ((rgb bitShift: -12) bitAnd: 16r00F0)) bitOr:
             ((rgb bitShift: -6) bitAnd: 16r000F).
        ^ val = 0 ifTrue: [bitBltFakeBlack] ifFalse: [val]].

    d = 9 ifTrue: [  "for indexing a color map with 3 bits per color 
component"
        val := (((rgb bitShift: -21) bitAnd: 16r01C0) bitOr:
             ((rgb bitShift: -14) bitAnd: 16r0038)) bitOr:
             ((rgb bitShift: -7) bitAnd: 16r0007).
        ^ val = 0 ifTrue: [bitBltFakeBlack] ifFalse: [val]].

    self error: 'unknown pixel depth: ', d printString

! !

!SimpleMIDIPort class methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/25/2013 10:29'!
portDescription: portNum
	"Answer a string indicating the directionality of the given MIDI port."
	"(0 to: SimpleMIDIPort primPortCount - 1) collect:
		[:i | SimpleMIDIPort portDescription: i]"

	| portName dir |
	portName := (self primPortNameOf: portNum) "convertFromSystemString".
	dir := self primPortDirectionalityOf: portNum.
	dir = 1 ifTrue: [^ portName, ' (in)'].
	dir = 2 ifTrue: [^ portName, ' (out)'].
	dir = 3 ifTrue: [^ portName, ' (in/out)'].
	^ self error: 'unknown MIDI port directionality'
! !

!MIDIInputParser methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/21/2013 11:25'!
processMIDIData
	"Process all MIDI data that has arrived since the last time this method was executed. This method should be called frequently to process, filter, and timestamp MIDI data as it arrives."

	| bytesRead |
	[(bytesRead := midiPort readInto: rawDataBuffer) > 0] whileTrue: [
		timeNow := (midiPort bufferTimeStampFrom: rawDataBuffer)" - startTime".
		5 to: bytesRead do: [:i | self processByte: (rawDataBuffer at: i)]].
! !

!PolygonMorph methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:15'!
removeShortEndSegmentsStepping: something

	"used as a hook for on-the-fly conversion from a connector v1.9 to a connector 2.2"

	self maybeBecomeNCLineMorph.
	self removeShortEndSegmentsStepping: something! !

!FormCanvas methodsFor: '*People-spfa-fixes' stamp: 'spfa 4/30/2011 22:51'!
reset

	origin := 0@0.							"origin of the top-left corner of this cavas"
	clipRect := (0@0 corner: 100000@100000).		"default clipping rectangle"
	self shadowColor: nil.! !

!TestBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 4/19/2010 14:49'!
runButtonColor

	^Color greenYellow! !

!SmalltalkImage methodsFor: '*People-spfa-fixes' stamp: 'spfa 9/6/2012 21:08'!
saveAs
	"Put up the 'saveAs' prompt, obtain a name, and save the image  under that new name."
	
	self saveAs: (self getFileNameFromUser ifNil: [^ self])
! !

!MIDIFileReader class methodsFor: '*People-spfa-fixes' stamp: 'spfa 8/31/2013 16:38'!
standardMIDIInstrumentNames
	"Answer an array of Standard MIDI instrument names."

	^ #(
		'Grand Piano'
		'Bright Piano'
		'Electric Grand Piano'
		'Honky-tonk Piano'
		'Electric Piano 1'
		'Electric Piano 2'
		'Harpsichord'
		'Clavichord'
		'Celesta'
		'Glockenspiel'
		'Music Box'
		'Vibraphone'
		'Marimba'
		'Xylophone'
		'Tubular Bells'
		'Dulcimer'
		'Drawbar Organ'
		'Percussive Organ'
		'Rock Organ'
		'Church Organ'
		'Reed Organ'
		'Accordion'
		'Harmonica'
		'Tango Accordion'
		'Nylon Guitar'
		'Steel Guitar'
		'Electric Guitar 1'
		'Electric Guitar 2'
		'Electric Guitar 3'
		'Overdrive Guitar'
		'Distorted Guitar'
		'Guitar Harmonics'
		'Acoustic Bass'
		'Electric Bass 1'
		'Electric Bass 2'
		'Fretless Bass'
		'Slap Bass 1'
		'Slap Bass 2'
		'Synth Bass 1'
		'Synth Bass 2'
		'Violin'
		'Viola'
		'Cello'
		'Contrabass'
		'Tremolo Strings'
		'Pizzicato Strings'
		'Orchestral Harp'
		'Timpani'
		'String Ensemble 1'
		'String Ensemble 2'
		'Synth Strings 1'
		'Synth Strings 2'
		'Choir Ahhs'
		'Choir Oohs'
		'Synth Voice'
		'Orchestra Hit'
		'Trumpet'
		'Trombone'
		'Tuba'
		'Muted Trumpet'
		'French Horn'
		'Brass Section'
		'Synth Brass 1'
		'Synth Brass 2'
		'Soprano Sax'
		'Alto Sax'
		'Tenor Sax'
		'Baritone Sax'
		'Oboe'
		'English Horn'
		'Bassoon'
		'Clarinet'
		'Piccolo'
		'Flute'
		'Recorder'
		'Pan Flute'
		'Blown Bottle'
		'Shakuhachi'
		'Whistle'
		'Ocarina'
		'Lead 1 (square)'
		'Lead 2 (sawtooth)'
		'Lead 3 (calliope)'
		'Lead 4 (chiff)'
		'Lead 5 (charang)'
		'Lead 6 (voice)'
		'Lead 7 (fifths)'
		'Lead 8 (bass+lead)'
		'Pad 1 (new age)'
		'Pad 2 (warm)'
		'Pad 3 (polysynth)'
		'Pad 4 (choir)'
		'Pad 5 (bowed)'
		'Pad 6 (metallic)'
		'Pad 7 (halo)'
		'Pad 8 (sweep)'
		'FX 1 (rain)'
		'FX 2 (soundtrack)'
		'FX 3 (crystals)'
		'FX 4 (atmosphere)'
		'FX 5 (brightness)'
		'FX 6 (goblins)'
		'FX 7 (echoes)'
		'FX 8 (sci-fi)'
		'Sitar'
		'Banjo'
		'Shamisen'
		'Koto'
		'Kalimba'
		'Bagpipe'
		'Fiddle'
		'Shanai'
		'Tinkle Bell'
		'Agogo'
		'Steel Drum'
		'Woodblock'
		'Taiko Drum'
		'Melodic Tom'
		'Synth Drum'
		'Reverse Cymbal'
		'Guitar Fret Noise'
		'Breath Noise'
		'Seashore'
		'Bird Tweet'
		'Telephone Ring'
		'Helicopter'
		'Applause'
		'Gunshot')
! !

!Rectangle methodsFor: '*People-spfa-fixes' stamp: 'spfa 11/18/2009 18:16'!
storeOn: aStream 
	
	aStream nextPut: $(.
	self printOn: aStream.
	aStream nextPut: $)! !

!TestBrowser methodsFor: '*People-spfa-fixes' stamp: 'spfa 4/19/2010 14:07'!
systemCategoryList

	^filterMode 
		ifFalse: 
			[systemCategoryList := (systemOrganizer categories 
						select: [:each | self pattern match: each]) asArray]
		ifTrue: 
			[systemCategoryList ifNil: 
					[| testCategories |
					testCategories := self allTestCaseClasses collect: [:each | each category].
					systemCategoryList := (testCategories asSet
								select: [:each | self pattern match: each]) asArray]
				ifNotNil: [systemCategoryList]]! !
