SystemOrganization addCategory: #StringFormat!


TestCase subclass: #StringFormatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StringFormat'!

Object subclass: #StringFormat
	instanceVariableNames: 'repKey simpleRepKey repDictionary simpleRepDictionary inliners macros'
	classVariableNames: 'Help Macros Replacements SimpleReplacements'
	poolDictionaries: ''
	category: 'StringFormat'!

StringFormat class
	instanceVariableNames: ''!

StringFormatTest class
	instanceVariableNames: ''!


!StringFormat commentStamp: 'spfa 12/6/2005 19:06' prior: 0!
an extensible format utility

-- usage:

StringFormat format: aString 
(aString sformat)
StringFormat format: anArray into: aString
(aString sformat: anArray)

there are two kinds of keywords for replacement:
% keywords require an argument (use #sformat: or #format:into:)
& keywords have no argument (use #sformat or #format)

all keywords can have a numeric prefix, although most of them do not use it. see examples below.
StringFormat help 	"doIt"				"describe all known keywords"


-- system-wide extensibility:

an object can add keywords by implementing #registerStringFormats in its class-side
this method simply have to add new entries to the StringFormat dictionaries:

StringFormat replacementsDictionary
StringFormat simpleReplacementsDictionary
StringFormat helpDictionary

(see StringFormat>>#populateDictionaries for how to do so)

StringFormat browseDefinitions 	"show all implementors of #registerStringFormats"


-- examples:

'time is now:&t&T' sformat  "printIt" 		"simple replacements (no arguments)"

'I tell you %s+%s is %s' sformat: {7. 2. 8} "printIt" 			"replacements with arguments"

'I tell you %s[6+1]+%s[2] is %s[4*2]' sformat "printIt" 		"replacements with inlined arguments"

'e is about %5p' sformat: Float e. "printIt"					"one replacements with argument"

'e is about %5p[Float e]' sformat "printIt"						"one replacements with inlined argument"

'it is now {{Time now}}' sformat "printIt"						"{{...}} is a macro for %s[...]"

(1 to: 8) inject: '' into: [:s :i | s, ('&&r&&%st%%%sp' sformat: {i . i}) sformat: Float pi] "printIt"

'&rat time &T&r&t(use &&T for %q)&rI say %s is about %5f&r&t(use %%Nf for precision)' sformat: {'Time now' . Float pi . Float pi} "printIt"


-- customization

while the #sformat method used in the above example use the class side of StringFormat, it is also possible to create instances where everything from the characters making the keywords to macros and dictionaries can be replaced with specific values.

examples:

(StringFormat new inliners: '<>') format: '%s<1+1>'

(StringFormat new macros: {'['->'%s['}) format: '{{1+1}}[1+1]'

(StringFormat new macrosAddFirst: ('['->'%s[')) format: '{{1+1}}[1+1]'

(StringFormat new replacementKey: $?) format: '?3f[Float pi]'

(StringFormat new simpleReplacementKey: $.) format: 'it is now .T'


!
]style[(2310)!

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
actualTestContext
	| a b |
	a := 1.
	b := 2.

	self assert:  '%s[a]' sformat =  '1'.
	self assert:  '{{b}}' sformat =  '2'.
	self assert: '3' = '{{a + b}}' sformat.
	self assert: '1 + 2 = 3' = '{{a}} + {{b}} = {{a + b}}' sformat.
	self assert: 'CHF: 2.00' = 'CHF: {{b printShowingDecimalPlaces: 2}}' sformat.


! !

!StringFormat class methodsFor: 'utilities' stamp: 'spfa 11/18/2009 18:16'!
browseDefinitions

	"StringFormat browseDefinitions" 
	
	SystemNavigation default browseAllImplementorsOf: #registerStringFormats! !

!StringFormat class methodsFor: 'utilities' stamp: 'spfa 11/18/2009 18:16'!
checkUpExternalDefinitions

	"StringFormat checkUpExternalDefinitions"

	(Object allSubclasses select: [:class | class respondsTo: #registerStringFormats])
		do: [:class | class registerStringFormats]
	! !

!StringFormat class methodsFor: 'class initialization' stamp: 'spfa 11/18/2009 18:16'!
defineMacros

	Macros := Array
					with: '{{' -> '%s['
					with: '}}' -> ']'
	! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
expandMacros: aStringOrText

	^ self macros inject: aStringOrText into: [:str : assoc | str copyReplaceAll: assoc key with: assoc value].
! !

!StringFormat class methodsFor: 'invocation' stamp: 'spfa 11/18/2009 18:16'!
format: aString

	^ self new format: #() into: aString context: thisContext sender

! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
format: aString

	^ self format: #() into: aString context: thisContext sender

! !

!StringFormat class methodsFor: 'invocation' stamp: 'spfa 11/18/2009 18:16'!
format: objectOrList into: aString

	^ self new format: objectOrList into: aString context: thisContext sender

! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
format: objectOrList into: aString

	^ self format: objectOrList into: aString context: thisContext sender

! !

!StringFormat class methodsFor: 'invocation' stamp: 'spfa 11/18/2009 18:16'!
format: objectOrList into: aStringOrText context: aContext

	^ self new format: objectOrList into: aStringOrText context: aContext

! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
format: objectOrList into: aStringOrText context: aContext

	| str args |

	str := self expandMacros: aStringOrText.

	str := self replaceAllIn: str 
			using: self simpleReplacementsDictionary 
			character: self simpleReplacementKey
			values: #()
			context: aContext.

	(objectOrList isCollection & objectOrList isString not)
		ifTrue: [args := objectOrList]
		ifFalse: [args := {objectOrList}].

	str := self replaceAllIn: str 
			using: self replacementsDictionary 
			character: self replacementKey
			values: args
			context: aContext.

	aStringOrText isSymbol ifTrue: [^ str asString asSymbol].
	^ str as: aStringOrText species

! !

!StringFormat class methodsFor: 'help' stamp: 'spfa 11/18/2009 18:16'!
help

	"StringFormat help"

	self helpText openInWorkspaceWithTitle: 'StringFormat definitions'
	! !

!StringFormat class methodsFor: 'help' stamp: 'spfa 11/18/2009 18:16'!
helpDictionary 

	"StringFormat helpDictionary explore"

	^ Help
	! !

!StringFormat class methodsFor: 'help' stamp: 'spfa 11/18/2009 18:16'!
helpText

	| hs |

	hs := TextStream on: ''.
	self helpDictionary keysSortedSafely do: [:k |
		hs nextPutAll: k; space; space; space ; nextPutAll: (self helpDictionary at: k); cr].
	^ hs contents
	
! !

!Integer methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:10'!
hexString: extent

	| hs |
	
	hs := '0000000000', self printStringHex asLowercase.
	^ hs copyFrom: hs size + 1 - extent to: hs size! !

!StringFormat class methodsFor: 'class initialization' stamp: 'spfa 11/18/2009 18:16'!
initialize

	"StringFormat initialize"
	
	Replacements := Dictionary new.
	SimpleReplacements := Dictionary new.
	Help := Dictionary new.

	self populateDictionaries; defineMacros; checkUpExternalDefinitions

	! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
inliners

	^ '[]'! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
inliners

	^ inliners ifNil: [self class inliners]! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
inliners: aString

	inliners := aString! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
macros

	^ Macros
	! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
macros

	^ macros  ifNil: [self class macros ]! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
macros: anArray

	macros := anArray! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
macrosAddFirst: anAssociation

	self macros: {anAssociation},self macros! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
macrosAddLast: anAssociation

	self macros: self macros, {anAssociation}! !

!StringFormat class methodsFor: 'class initialization' stamp: 'spfa 11/18/2009 18:16'!
populateDictionaries

	{
	 	$s -> [:o :n | o ].
	 	$k -> [:o :n | o printString].
	 	$q -> [:o :n | o asString surroundedBySingleQuotes].
	 	$Q -> [:o :n | o asString surroundedByDoubleQuotes].
	 	$C -> [:o :n | o asString capitalized].
	 	$U -> [:o :n | o asString translateToUppercase].
	 	$l -> [:o :n | o asString translateToLowercase].
	 	$Y -> [:o :n | o yesNoString].
	 	$? -> [:o :n | (o first ifFalse: [''] ifTrue: [o second value])].
	 	$d -> [:o :n | (o * 1000) milliSeconds].
	 	$i -> [:o :n | o asString truncateTo: n].
		$p -> [:o :n | o precision: n].
		$f -> [:o :n | o printShowingDecimalPlaces: n].
		$h -> [:o :n | o hexString: n]
	} do:
		[:a | Replacements add: a].

	{
	 	$T -> [:n | Time now].
	 	$t -> [:n | String new: n withAll: Character tab].
	 	$r -> [:n | String new: n withAll: Character cr].
	} do:
		[:a | SimpleReplacements add: a].

	{
	 	'%s' -> 'generic replacement'.
	 	'%k' -> '#symbol'.
	 	'%q' -> 'surround by simple quotes'.
	 	'%Q' -> 'surround by double quotes'.
	 	'%C' -> 'capitalize'.
	 	'%U' -> 'uppercase'.
	 	'%l' -> 'lowercase'.
	 	'%Y' -> '''<yes>'' or ''<no>'' (for Boolean)'.
	 	'%?' -> 'on {test.object}: act as %s for object or is ignored depending on test'.
	 	'%d' -> 'as duration (in seconds)'.
	 	'%Ni' -> 'generic replacement truncated at N characters'.
	 	'%%' -> 'a plain %'.
	 	'&&' -> 'a plain &'.
		'&Nt' -> 'N times Character tab'.
		'&Nr' -> 'N times Character cr'.
		'&T' -> 'Time now'.
		'%Np' -> 'with precision N'.
		'%Nh' -> 'hexadecimal with N figures'
	} do:
		[:a | Help add: a].
! !

!Collection methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:08'!
precision: aNumber
	
	^ self collect: [:ea | ea precision: aNumber]! !

!Dictionary methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:09'!
precision: aNumber

	| dict |

	dict := Dictionary new.
	self keysAndValuesDo:
		 [:k :v | dict add: k -> (v precision: aNumber)].
		
	^ dict! !

!Float methodsFor: '*StringFormat' stamp: 'spfa 9/20/2011 09:35'!
precision: aNumber

	| pten |

	self isInfinite ifTrue: [^ self].

	pten := 10 raisedTo: aNumber.
	^ (pten * self) rounded /// pten! !

!Fraction methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:10'!
precision: aNumber

	^ self asFloat precision: aNumber! !

!Integer methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:10'!
precision: aNumber

	^ self ! !

!LineSegment methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:11'!
precision: aNumber

	self from: (start precision: aNumber) to: (end precision: aNumber)! !

!Object methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:15'!
precision: aNumber

	^ self! !

!Point methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:15'!
precision: anInteger

	^ (self x precision: anInteger) @ (self y precision: anInteger) ! !

!Rectangle methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:16'!
precision: anInteger

	^ (self origin precision: anInteger) corner: (self corner precision: anInteger)
! !

!ScaledDecimal methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:16'!
precision: aNumber

	^ self asFloat precision: aNumber! !

!String methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:16'!
precision: aNumber

	^ self contractTo: aNumber! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
replaceAllIn: aStringOrText using: aDictionary character: aChar values: anArray context: aContext

	| tokens blah args cs compiler iopc iclc weirdo |

	blah := aStringOrText.
	(blah includes: aChar) ifFalse: [^ blah].

	args := anArray.
	cs := aChar asString.
	iopc := self inliners first.
	iclc := self inliners second.
	weirdo := self weirdWord.
	compiler := aContext sender class compilerClass new.

	blah := blah copyReplaceAll: cs, cs with: weirdo.
	tokens := blah asString findTokens: cs.
	(blah first = aChar)
			ifTrue: [blah := '']
			ifFalse: [blah := tokens first. tokens := tokens allButFirst].
	args size = tokens size
		ifFalse: [args := args, (Array new: tokens size - args size)].

	^ (blah asText, (Text streamContents: [:stream | 
		tokens with: args do: [:s :o |	
			| n no ki  block follow |
			ki := s findFirst: [:m | m isDigit not].	
			n := (s first: ki - 1) asInteger ifNil: [1].
			no := o.

			"inlined argument - note that it still consumes o"
			(s at: ki + 1 ifAbsent: [nil]) = iopc 
				ifTrue:
				[	| end |
					end := s indexOf: iclc startingAt: ki +2.
					no ifNotNil: [self error: 'sformat values conflict'].
					no := compiler evaluate: (s copyFrom: ki+2 to: end-1)
						in: aContext
						to: aContext receiver 
						notifying: nil 
						ifFail: [ self error: 'Compile error' ]
						logged: false.
					follow := s allButFirst: end
				]
			ifFalse: [follow := s allButFirst: ki].
	
			block := aDictionary at: (s at: ki) ifAbsent: 
					[
						self class checkUpExternalDefinitions. "dynamic updating of the dictionary"
						self simpleReplacementsDictionary at: s first 
							ifAbsent: [self error: ('%s unknown sformat' sformat: (s at: ki))]
					].
			block numArgs = 1
				ifTrue: [stream nextPutAll: (block value: n) asString]
				ifFalse: [stream nextPutAll: (block value: no value: n) asString].
			stream nextPutAll: follow]
	]))
	copyReplaceAll: weirdo with: cs

! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
replacementKey

	^ $%! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
replacementKey

	^ repKey ifNil: [self class replacementKey]! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
replacementKey: aCharacter

	repKey := aCharacter! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
replacementsDictionary 

	"StringFormat replacementsDictionary explore"

	^ Replacements
	! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
replacementsDictionary 

	^ repDictionary  ifNil: [self class replacementsDictionary ]! !

!String methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:16'!
sformat

	^ StringFormat format: #() into: self context: thisContext sender
	! !

!Text methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:17'!
sformat

	^ StringFormat format: #() into: self context: thisContext sender! !

!String methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:16'!
sformat: objectOrList

	"replace the keywords in self with the corresponding representation of the objects in objectOrList. see keywords definitions:

	StringFormat help
	"	
	^ StringFormat format: objectOrList into: self context: thisContext sender
	! !

!Text methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:17'!
sformat: objectOrList

	"replace the keywords in self with the corresponding representation of the objects in objectOrList. see keywords definitions:

	StringFormat help
	"	
	^ StringFormat format: objectOrList into: self context: thisContext sender! !

!TranscriptStream methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:17'!
sformat: anArray into: aString

	self show: (aString sformat: anArray)! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
simpleReplacementKey

	^ $&! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
simpleReplacementKey

	^ simpleRepKey ifNil: [self class simpleReplacementKey]! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
simpleReplacementKey: aCharacter

	simpleRepKey := aCharacter! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
simpleReplacementsDictionary

	"StringFormat simpleReplacementsDictionary explore"

	^ SimpleReplacements
	! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
simpleReplacementsDictionary 

	^ simpleRepDictionary  ifNil: [self class simpleReplacementsDictionary ]! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testBasics

	self assert: ('%s-%s+%s' sformat: #(1 2 3)) = '1-2+3'.
	self assert: ('%10i' sformat: 'lalallahiloulalalalhiloulahiloulalalalhiloula') = 'lalallahil'.
	self assert: ('%10p' sformat: 'lalallahiloulalalalhiloulahiloulalalalhiloula') = 'lala...ula'.
	self assert: ('%U' sformat: 'lalalla') = 'LALALLA' .
	self assert: ('a duration of %d' sformat: 36125) = 'a duration of 0:10:02:05'.
	self assert: ('answer %Y/%Y' sformat: {true . false}) = 'answer <yes>/<no>'.
	self assert: ('animals:%?%?' sformat: {{false . ' dog'}.{true . ' cat'}}) = 'animals: cat'.
	self assert: ('animals:%?%?' sformat: {{true . ' dog'}.{true . ' cat'}}) = 'animals: dog cat'.
	self assert: ('animals:%?%?' sformat: {{true . ' dog'}.{false . [nil]}}) = 'animals: dog'.

	self assert: 'time is &T' sformat = ('time is ', Time now asString).
	self assert: '&r&t' sformat = (String cr, String tab).
	self assert: '&3r&2t' sformat = (String cr, String cr, String cr, String tab, String tab).

	self assert: ('45 = %2h' sformat: 45) = '45 = 2d'.
	self assert: ('45 = %8h' sformat: 45) = '45 = 0000002d'.

	self assert: ('%3p' sformat: 2) = '2'.
	self assert: ('%3p' sformat: 2.7895) = '2.79'.
	self assert: ('%3f' sformat: 2) = '2.000'.
	self assert: ('%3f' sformat: 2.7895) = '2.790'.
! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testContext

" fails in a forked process (and notably with a GUI)
... this could be 'fixed' (?) by having MessageAsTempNode>>scope return 0 or 1

(StringFormatTest selector: #actualTestContext) run hasPassed
"! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testDouble

	self assert: ((1 to: 8) inject: '' 
					into: [:s :i | s, ('&&r&&%st%%%sp' sformat: {i . i}) sformat: Float pi]) =
 '
	3.1
		3.14
			3.142
				3.1416
					3.14159
						3.141593
							3.1415927
								3.14159265'
! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testInlinedArguments

	self assert: ('%s[1]+%s[2]=%s[3]' sformat) = '1+2=3'.
	self assert: ('{{1}}+{{2}}={{1+2}}' sformat) = '1+2=3'.
	self assert: ('%10i' sformat: 'lalallahiloulalalalhiloulahiloulalalalhiloula') = 'lalallahil'.
	self assert: ('%10p[''lalallahiloulalalalhiloulahiloulalalalhiloula'']' sformat) = 'lala...ula'.
	self assert: ('a duration of %d[36125]' sformat) = 'a duration of 0:10:02:05'.
	self assert: ('animals:%?[{false . '' dog''}]%?[{true . '' cat''}]' sformat) = 'animals: cat'.
	self assert: ('45 = %8h[45]' sformat) = '45 = 0000002d'.

! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testMacros

	self assert: ((StringFormat new macros: {'t'->'&T' . '['->'%s['}) format: '[1+1]t') = ( '2', Time now asString).

	self assert: ((StringFormat new macros: {'['->'%s['}) format: '{{1+1}}[1+1]') = '{{1+1}}2'.
	self assert: ((StringFormat new macrosAddFirst: ('['->'%s[')) format: '{{1+1}}[1+1]') = '22'.
	self assert: ((StringFormat new macrosAddLast: (']['->'+')) format: '{{1+1}}[1+1]') = '4'.! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testSelfSend

	self assert: 'Im coming from: StringFormatTest' = 'Im coming from: {{self class name}}' sformat
! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testSymbol

	self assert: #'{{5+4}}' sformat isSymbol! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testText

	self assert: 'aef' asText sformat isText! !

!StringFormatTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
testVariations

	self assert: ((StringFormat new inliners: '<>') format: '%3f<Float pi>') = '3.141'.
	self assert: ((StringFormat new inliners: '||') format: '%3f|Float pi|') = '3.141'.
	self assert: ((StringFormat new replacementKey: $$) format: '$3f[Float pi]') = '3.141'.
	self assert: ((StringFormat new simpleReplacementKey: $.) format: '.T') = Time now asString.
! !

!StringFormat class methodsFor: 'default' stamp: 'spfa 11/18/2009 18:16'!
weirdWord

	^ '-ZnutP$-'! !

!StringFormat methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:16'!
weirdWord

	^ self class weirdWord! !

!False methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:09'!
yesNoString

	^ '<no>'! !

!True methodsFor: '*StringFormat' stamp: 'spfa 11/18/2009 18:17'!
yesNoString

	^ '<yes>'! !
StringFormat initialize!
